/*
    This file is part of PicoScope.

    PicoScope is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PicoScope is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PicoScope.  If not, see <http://www.gnu.org/licenses/>.

    Original Written by Tobie de Beer for personal use
*/

#include "picoscope.h"
#include "ui_picoscope.h"

short PS_hand = NULL;
short Samples = 1;
double TimeDiv = 100.0;
double VoltDiv = 32024.0/20.0;
QString TimeAxis = "Time(S)";
QString VoltAxis = "Voltage(V)";
double TimeRange = 1.0;
double VoltRange = 20.0;
short TimeGrid = 10;
short VoltSet = PS2000_1V;
int TimerSet = 100;
short SetDC = 1;
short TrigDir = 0;
short TrigChan = 0;
short TrigLevel = 0;

PicoScope::PicoScope(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::PicoScope)
{
    ui->setupUi(this);
    PS_hand = ps2000_open_unit();
    if (PS_hand == NULL ) {
        ui->Exit_Now->setText("quiting!! no Picoscope detected");
        sleep(2);
        exit(1);
    }
      else {
        ui->Exit_Now->hide();
        ui->customPlot->addGraph();
        Timer = new QTimer(this);
        Timer->connect(Timer,SIGNAL(timeout()),this,SLOT(on_Timer()));
        on_new_Config();
    }
}

PicoScope::~PicoScope()
{
    if (PS_hand != NULL) {
        ps2000_stop(PS_hand);
        ps2000_close_unit(PS_hand);
        Timer->stop();
        delete Timer;
    }
    delete ui;
}

void PicoScope::on_Timer() // this is where everyting happens!
{
    QVector<double> x(8000), y(8000);
    long temp;
    short a[8000],overflow;
    if (ps2000_ready(PS_hand)!=0) {
        ps2000_get_values(PS_hand, a, NULL, NULL, NULL, &overflow, Samples );
        for (int i=0; i<Samples; ++i) {
            x[i] = i/TimeDiv;
            y[i] = a[i]/VoltDiv;
        }
        ui->customPlot->graph(0)->setData(x, y);
        ui->customPlot->replot();
        ps2000_run_block(PS_hand, Samples, TimeGrid, 1, &temp);
        Timer->setInterval(TimerSet);
    }
      else
        Timer->setInterval(1);

}

void PicoScope::on_Light_toggled(bool checked)
{
    ps2000_set_light(PS_hand,checked);
}

void PicoScope::on_new_Config()
{
    long temp;

    ps2000_stop(PS_hand);
    ps2000_flash_led(PS_hand);

    if (ui->RB_20V->isChecked()) {
        VoltSet = PS2000_20V;
        VoltAxis = "Voltage(V)";
        VoltRange = 20.0;
        VoltDiv = 32024.0/20.0;
    }
    if (ui->RB_10V->isChecked()) {
        VoltSet = PS2000_10V;
        VoltAxis = "Voltage(V)";
        VoltRange = 10.0;
        VoltDiv = 32024.0/10.0;
    }
    if (ui->RB_5V->isChecked()) {
        VoltSet = PS2000_5V;
        VoltAxis = "Voltage(V)";
        VoltRange = 5.0;
        VoltDiv = 32024.0/5.0;
    }
    if (ui->RB_2V->isChecked()) {
        VoltSet = PS2000_2V;
        VoltAxis = "Voltage(V)";
        VoltRange = 2.0;
        VoltDiv = 32024.0/2.0;
    }
    if (ui->RB_1V->isChecked()) {
        VoltSet = PS2000_1V;
        VoltAxis = "Voltage(V)";
        VoltRange = 1.0;
        VoltDiv = 32024.0/1.0;
    }
    if (ui->RB_500mV->isChecked()) {
        VoltSet = PS2000_500MV;
        VoltAxis = "Voltage(mV)";
        VoltRange = 500.0;
        VoltDiv = 32024.0/500.0;
    }
    if (ui->RB_200mV->isChecked()) {
        VoltSet = PS2000_200MV;
        VoltAxis = "Voltage(mV)";
        VoltRange = 200.0;
        VoltDiv = 32024.0/200.0;
    }
    if (ui->RB_100mV->isChecked()) {
        VoltSet = PS2000_100MV;
        VoltAxis = "Voltage(mV)";
        VoltRange = 100.0;
        VoltDiv = 32024.0/100.0;
    }
    if (ui->RB_DC->isChecked())
        SetDC = 1;
    if (ui->RB_AC->isChecked())
        SetDC = 0;

    ps2000_set_channel(PS_hand,PS2000_CHANNEL_A,1,SetDC,VoltSet);
    ui->customPlot->yAxis->setLabel(VoltAxis);
    ui->customPlot->yAxis->setRange(-VoltRange, VoltRange);

    if (ui->RB_Rising->isChecked()) {
        TrigChan = PS2000_CHANNEL_A;
        TrigDir = PS2000_RISING;
        TrigLevel = ui->TrigVolt->value();
    }
    if (ui->RB_Falling->isChecked()) {
        TrigChan = PS2000_CHANNEL_A;
        TrigDir = PS2000_RISING;
        TrigLevel = ui->TrigVolt->value();
    }
    if (ui->RB_Trig_None->isChecked()) {
        TrigChan = PS2000_NONE;
        TrigDir = PS2000_RISING;
        TrigLevel = 0;
    }

    ps2000_set_trigger(PS_hand, TrigChan, TrigLevel, TrigDir, 0, 0);

    if (ui->RB_50S->isChecked()) {
        TimeGrid = 19;
        TimeAxis = "Time(S)";
        TimeRange = 50.0;
        Samples = 4768;
        TimeDiv = 4768.0/50.0;
        TimerSet = 50000;
    }
    if (ui->RB_20S->isChecked()) {
        TimeGrid = 17;
        TimeAxis = "Time(S)";
        TimeRange = 20.0;
        Samples = 7629;
        TimeDiv = 7629.0/20.0;
        TimerSet = 20000;
    }
    if (ui->RB_10S->isChecked()) {
        TimeGrid = 16;
        TimeAxis = "Time(S)";
        TimeRange = 10.0;
        Samples = 7629;
        TimeDiv = 7629.0/10.0;
        TimerSet = 10000;
    }
    if (ui->RB_5S->isChecked()) {
        TimeGrid = 15;
        TimeAxis = "Time(S)";
        TimeRange = 5.0;
        Samples = 7629;
        TimeDiv = 7629.0/5.0;
        TimerSet = 5000;
    }
    if (ui->RB_2S->isChecked()) {
        TimeGrid = 14;
        TimeAxis = "Time(S)";
        TimeRange = 2.0;
        Samples = 6104;
        TimeDiv = 6104.0/2.0;
        TimerSet = 2000;
    }
    if (ui->RB_1S->isChecked()) {
        TimeGrid = 13;
        TimeAxis = "Time(S)";
        TimeRange = 1.0;
        Samples = 6104;
        TimeDiv = 6104.0/1.0;
        TimerSet = 1000;
    }
    if (ui->RB_500mS->isChecked()) {
        TimeGrid = 12;
        TimeAxis = "Time(mS)";
        TimeRange = 500.0;
        Samples = 6104;
        TimeDiv = 6104.0/500.0;
        TimerSet = 500;
    }
    if (ui->RB_200mS->isChecked()) {
        TimeGrid = 11;
        TimeAxis = "Time(mS)";
        TimeRange = 200.0;
        Samples = 4883;
        TimeDiv = 4883.0/200.0;
        TimerSet = 200;
    }
    if (ui->RB_100mS->isChecked()) {
        TimeGrid = 10;
        TimeAxis = "Time(mS)";
        TimeRange = 100.0;
        Samples = 4883;
        TimeDiv = 4883.0/100.0;
        TimerSet = 100;
    }
    if (ui->RB_50mS->isChecked()) {
        TimeGrid = 9;
        TimeAxis = "Time(mS)";
        TimeRange = 50.0;
        Samples = 4883;
        TimeDiv = 4883.0/50.0;
        TimerSet = 50;
    }
    if (ui->RB_20mS->isChecked()) {
        TimeGrid = 7;
        TimeAxis = "Time(mS)";
        TimeRange = 20.0;
        Samples = 7813;
        TimeDiv = 7813.0/20.0;
        TimerSet = 20;
    }
    if (ui->RB_10mS->isChecked()) {
        TimeGrid = 6;
        TimeAxis = "Time(mS)";
        TimeRange = 10.0;
        Samples = 7813;
        TimeDiv = 7813.0/10.0;
        TimerSet = 10;
    }
    if (ui->RB_5mS->isChecked()) {
        TimeGrid = 5;
        TimeAxis = "Time(mS)";
        TimeRange = 5.0;
        Samples = 7813;
        TimeDiv = 7813.0/5.0;
        TimerSet = 5;
    }
    if (ui->RB_2mS->isChecked()) {
        TimeGrid = 4;
        TimeAxis = "Time(mS)";
        TimeRange = 2.0;
        Samples = 6250;
        TimeDiv = 6250.0/2.0;
        TimerSet = 2;
    }
    if (ui->RB_1mS->isChecked()) {
        TimeGrid = 3;
        TimeAxis = "Time(mS)";
        TimeRange = 1.0;
        Samples = 6250;
        TimeDiv = 6250.0/1.0;
        TimerSet = 1;
    }
    if (ui->RB_500uS->isChecked()) {
        TimeGrid = 2;
        TimeAxis = "Time(uS)";
        TimeRange = 500.0;
        Samples = 6250;
        TimeDiv = 6250.0/500.0;
        TimerSet = 1;
    }
    if (ui->RB_200uS->isChecked()) {
        TimeGrid = 1;
        TimeAxis = "Time(uS)";
        TimeRange = 200.0;
        Samples = 5000;
        TimeDiv = 5000.0/200.0;
        TimerSet = 1;
    }
    if (ui->RB_100uS->isChecked()) {
        TimeGrid = 0;
        TimeAxis = "Time(uS)";
        TimeRange = 100.0;
        Samples = 5000;
        TimeDiv = 5000.0/100.0;
        TimerSet = 1;
    }
    if (ui->RB_50uS->isChecked()) {
        TimeGrid = 0;
        TimeAxis = "Time(uS)";
        TimeRange = 50.0;
        Samples = 2500;
        TimeDiv = 2500.0/50.0;
        TimerSet = 1;
    }
    if (ui->RB_20uS->isChecked()) {
        TimeGrid = 0;
        TimeAxis = "Time(uS)";
        TimeRange = 20.0;
        Samples = 1000;
        TimeDiv = 1000.0/20.0;
        TimerSet = 1;
    }
    if (ui->RB_10uS->isChecked()) {
        TimeGrid = 0;
        TimeAxis = "Time(uS)";
        TimeRange = 10.0;
        Samples = 500;
        TimeDiv = 500.0/10.0;
        TimerSet = 1;
    }
    if (ui->RB_5uS->isChecked()) {
        TimeGrid = 0;
        TimeAxis = "Time(uS)";
        TimeRange = 5.0;
        Samples = 250;
        TimeDiv = 250.0/5.0;
        TimerSet = 1;
    }
    if (ui->RB_2uS->isChecked()) {
        TimeGrid = 0;
        TimeAxis = "Time(uS)";
        TimeRange = 2.0;
        Samples = 100;
        TimeDiv = 100.0/2.0;
        TimerSet = 1;
    }


    ui->customPlot->xAxis->setLabel(TimeAxis);
    ui->customPlot->xAxis->setRange(0.0, TimeRange);

    ps2000_run_block(PS_hand, Samples, TimeGrid, 1, &temp);
    Timer->setInterval(TimerSet);
    Timer->start(TimerSet);
}

void PicoScope::on_RB_Rising_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_Falling_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_Trig_None_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_20V_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_10V_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_5V_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_2V_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_1V_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_500mV_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_200mV_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_100mV_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_AC_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_DC_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_2uS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_5uS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_10uS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_20uS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_50uS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_100uS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_200uS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_500uS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_1mS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_2mS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_5mS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_10mS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_20mS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_50mS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_100mS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_200mS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_500mS_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_1S_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_2S_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_5S_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_10S_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_20S_clicked()
{
    on_new_Config();
}

void PicoScope::on_RB_50S_clicked()
{
    on_new_Config();
}


void PicoScope::on_TrigVolt_sliderReleased()
{
    on_new_Config();
}
